define([
    "dojo/_base/declare",
    "dojo/dom-class",
    "dijit/registry",
    "dgrid/util/misc",
    "epi/dependency",
    // Parent class and mixins
    "dijit/_WidgetBase",
    "dijit/_CssStateMixin",
    "epi/shell/widget/_FocusableMixin",
    // List modules
    "dgrid/OnDemandList",
    "dgrid/Keyboard",
    "dgrid/extensions/DijitRegistry",
    // Project modules
    "epi-cms/project/Activity"
], function(
    declare,
    domClass,
    registry,
    misc,
    dependency,
    // Parent class and mixins
    _WidgetBase,
    _CssStateMixin,
    _FocusableMixin,
    // List modules
    OnDemandList,
    Keyboard,
    DijitRegistry,
    // Project modules
    Activity
) {

    // Create a custom list class for the activity feed.
    var ActivityList = declare([OnDemandList, Keyboard, DijitRegistry]);

    return declare([_WidgetBase, _CssStateMixin, _FocusableMixin], {
        // summary:
        //      A widget used to list activities.
        // tags:
        //      internal

        // baseClass: [protected] String
        //      The base CSS class of the widget, used to construct CSS classes
        //      to indicate widget state.
        baseClass: "epi-activity-feed",

        // noDataMessage: [public] String
        //      The message to display when there is no data returned by the server
        //      or when there is no query.
        noDataMessage: "",

        // query: [public] Object
        //      The object used to query the store for activities.
        query: null,

        // store: [readonly] dojo/store
        //      The store used to query for activities.
        store: null,

        postMixInProperties: function() {
            // summary:
            //      Called after the parameters to the widget have been read-in,
            //      but before the widget template is instantiated.
            // tags:
            //      protected

            this.inherited(arguments);

            // Resolve the store from the registry if it hasn't been injected.
            this.store = this.store || dependency.resolve("epi.storeregistry").get("epi.cms.activities");
        },

        buildRendering: function() {
            // summary:
            //      Build the list component to be shown within this widget.
            // tags:
            //      protected

            this.inherited(arguments);

            this.list = new ActivityList({
                removeRow: this._removeRow,
                renderRow: this._renderRow
            });

            this.domNode.appendChild(this.list.domNode);
        },

        startup: function() {
            // summary:
            //      Processing after the DOM fragment is added to the document. Also calls
            //      startup on the list.
            // tags:
            //      public

            this.inherited(arguments);
            this.list.startup();
        },

        onFocus: function() {
            // summary:
            //      Set focus to the project item list.
            // tags:
            //      public

            if (this.list) {
                this.list.focus();
            }
        },

        refresh: function() {
            // summary:
            //      Refreshes the list causing it to query the server for new data.
            // tags:
            //      public

            this.list.refresh();
        },

        _removeRow: function(rowElement, justCleanup) {
            // summary:
            //      Destroys the activity widget associated with the row as well as
            //      removing the row from the list.
            // rowElement: Object|DOMNode
            //      Object or element representing the row to be removed.
            // justCleanup: Boolean
            //      If true, the row element will not be removed from the DOM.
            // tags:
            //      private

            // This callback method runs in the context of the list.
            this.inherited(arguments);

            var element = rowElement.element || rowElement,
                widget = registry.byNode(element);

            if (widget) {
                widget.destroy(justCleanup);
            }
        },

        _renderRow: function(value) {
            // summary:
            //      Renders an activity widget for the row and returns its DOM node.
            // tags:
            //      private

            var activity = new Activity(value);

            // Style each row in the list as a card.
            domClass.add(activity.domNode, "epi-card");

            return activity.domNode;
        },

        _setNoDataMessageAttr: function(noDataMessage) {
            // summary:
            //      Sets the no data message property and updates the grid.
            // tags:
            //      protected

            this.noDataMessage = noDataMessage;

            this.list.set("noDataMessage", noDataMessage);
        },

        _setQueryAttr: function(query) {
            // summary:
            //      Sets the query property and updates the grid, debouncing if needed.
            // tags:
            //      protected

            this.query = query;

            // If there is no query then set the store to null so that the
            // grid is cleared of results without querying the server again.
            var store = query ? this.store : null;

            if (!this._queryFunction) {
                this._queryFunction = misc.debounce(this._setQueryCallback.bind(this), null, 200);
            }

            this._queryFunction(store, query);
        },

        _setQueryCallback: function(store, query) {
            // summary:
            //      Callback that sets the grid's query and updates the
            //      grid when debouncing has finished.
            // tags:
            //      private

            // The number of content references in the query determines how many items we are showing
            // activities for. Toggle the single item modifier class when this is one so that content
            // names on activities can be hidden with CSS.
            var references = query && query.contentReferences;
            domClass.toggle(this.domNode, "epi-activity-feed--single-item", references && references.length == 1);

            this.list.set("store", store, query);
        }
    });
});
